//============================================================================
//  aPCI-8815 for PCI Linux Device Driver
//  apci8815.h
//  Author  Shoichiro Ito(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date    2001/03/16
//  Rev.    1.0.0
//============================================================================


#ifndef __INC_APCI8815_H__
#define __INC_APCI8815_H__


//-----------------------------------------------------------
//  Macros to help debugging
//-----------------------------------------------------------

#undef PDEBUG		  /* undef it, just in case */
#ifdef APCI8815_DEBUG
#	ifdef __KERNEL__
		/* This one if debugging is on, and kernel space */
#		define PDEBUG(fmt, args...) printk(KERN_DEBUG "[APCI8815]: " fmt, ## args)
#	else
		/* This one for user space */
#		define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#	endif
#else
#	define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */


//-----------------------------------------------------------
//  Miscellaneous
//-----------------------------------------------------------

#define APCI8815_MAX_DEVICES	16	// ǥХ


//-----------------------------------------------------------
//  Sampling mode
//-----------------------------------------------------------
					// Start Sampling Transfer CLK
#define APCI8815_AD_MD_01	1	// soft	 single	  port	   ---
#define APCI8815_AD_MD_02	2	// soft	 single	  port	   EXTRG
#define APCI8815_AD_MD_03	3	// soft	 burst	  port	   timer
#define APCI8815_AD_MD_04	4	// soft	 burst	  port	   EXTRG


//-----------------------------------------------------------
//  Sampling status
//-----------------------------------------------------------

#define APCI8815_STS_SMP	0x01
#define APCI8815_STS_EOC	0x02
#define APCI8815_STS_BUF_HFULL	0x04
#define APCI8815_STS_BUF_FULL	0x08
#define APCI8815_STS_BUF_OVFLOW	0x10
#define APCI8815_STS_SMP_OVRUN	0x20

//-----------------------------------------------------------
//  Channel range
//-----------------------------------------------------------

#define APCI8815_CH_FIRST	0
#define APCI8815_CH_LAST	15


//-----------------------------------------------------------
//  Channel
//-----------------------------------------------------------

#define APCI8815_CH_01		0
#define APCI8815_CH_02		1
#define APCI8815_CH_03		2
#define APCI8815_CH_04		3
#define APCI8815_CH_05		4
#define APCI8815_CH_06		5
#define APCI8815_CH_07		6
#define APCI8815_CH_08		7
#define APCI8815_CH_09		8
#define APCI8815_CH_10		9
#define APCI8815_CH_11		10
#define APCI8815_CH_12		11
#define APCI8815_CH_13		12
#define APCI8815_CH_14		13
#define APCI8815_CH_15		14
#define APCI8815_CH_16		15


//-----------------------------------------------------------
//  Sampling cycle
//-----------------------------------------------------------

#define APCI8815_CYCL_MIN	12		// ꤹ륿ޤκǾ
#define APCI8815_CYCL_CI_MIN	45		// ꤹ륿ޤκǾ(ͥ륤󥯥)
#define APCI8815_CYCL_MAX	64000000	// ꤹ륿ޤκ


//-----------------------------------------------------------
//  Device I/O Defined
//-----------------------------------------------------------

// Input Ports

#define APCI8815_STR_PORT	0x00	// STRݡ
#define APCI8815_CD_PORT	0x02	// CDݡ

// Output ports

#define APCI8815_CTL_PORT	0x00	// CTLݡ
#define APCI8815_TW_PORT	0x02	// TWݡ
#define APCI8815_INTC_PORT	0x04	// INTCݡ

// Offset 0x00	    APCI8815_STR_PORT(I)

#define APCI8815_STR_CVDF	0x0001	// STRݡ ɤ߽Фǽ
#define APCI8815_STR_EXTRG	0x0020	// STRݡ ExTrg ξ
#define APCI8815_STR_EXIN0	0x0040	// STRݡ ExIn0 ξ
#define APCI8815_STR_EXIN1	0x0080	// STRݡ ExIn1 ξ

// Offset 0x00	    APCI8815_CTL_PORT(O)

#define APCI8815_CTL_RST	0x0000	// CTLݡ ꥻå
#define APCI8815_CTL_START	0x0001	// CTLݡ A/DѴ
#define APCI8815_CTL_INT	0x0004	// CTLݡ ޤγȯ
#define APCI8815_CTL_SMP	0x0006	// CTLݡ ޤA/DѴǡΰץ
#define APCI8815_CTL_CISW	0x0008	// CTLݡ ͥ륪ȥ󥯥
#define APCI8815_CTL_EXTSW	0x0010	// CTLݡ ȥꥬ
#define APCI8815_CTL_E0		0x0040	// CTLݡ ϼȿ0
#define APCI8815_CTL_E1		0x0080	// CTLݡ ϼȿ1

// Offset 0x04	    APCI8815_INTC_PORT(O)

#define APCI8815_INTC_RST	0x0000	// INTCݡ ꥻå

#define APCI8815_NOW_CH_ZERO	0x0000	// ͥ 0
#define APCI8815_PREV_CH_RESET	0xFFFF	// ͥ륤󥯥ѥꥻå


//-----------------------------------------------------------
//  Device I/O Control Codes
//-----------------------------------------------------------

#define APCI8815_IOC_MAGIC	'Z'

#define APCI8815_SET_AD_MD	_IOW (APCI8815_IOC_MAGIC, 0, apci8815_admd)
#define APCI8815_START_AD	_IO  (APCI8815_IOC_MAGIC, 1)
#define APCI8815_STOP_AD	_IO  (APCI8815_IOC_MAGIC, 2)
#define APCI8815_GET_AD_STS	_IOR (APCI8815_IOC_MAGIC, 3, unsigned long)
#define APCI8815_GET_EXIN	_IOR (APCI8815_IOC_MAGIC, 4, unsigned short)

#define APCI8815_IOC_MAXNR	4


// TODO:  桼ץꥱȤ̿Ѥǡ¤򤳤˵Ҥޤ

typedef struct _apci8815_admd
{
	int            buf_size;	// Хåե
	int            cycle;		// ץ󥰼
	unsigned short md;		// ץ󥰥⡼
        char	       ch_first;	// ץ󥰳ϥͥ (0  15)
	char	       ch_last;		// ץ󥰽λͥ (0  15)
} apci8815_admd, *papci8815_admd;


#endif // __INC_APCI8815_H__
