//============================================================================
//  aPCI-8815 for PCI Device Driver Diagnosis program
//  apci8815_diag.c
//  Author  Shoichiro Ito(ADTEK SYSTEM SCIENCE Co.,Ltd.)
//  Date    2001/03/16
//  Rev.    1.0.0
//  Note
//============================================================================

#include <stdio.h>	// used printf,perror...
#include <stdlib.h>	// used malloc,free
#include <unistd.h>	// used read
#include <sys/ioctl.h>
#include <term.h>
#include <string.h>	// used strerror
#include <errno.h>	// used errno

#include "apci8815ctl.h"


// input of command from user
static char line[256];


//--- Ǵؿ ----------------------------------------------

static void diag01_create(void);
static void diag02_close(void);
static void diag03_set_ad_md(void);
static void diag04_start_ad(void);
static void diag05_stop_ad(void);
static void diag06_read_ad(void);
static void diag07_get_ad_sts(void);
static void diag08_get_exin(void);
static void diag09_setup_int(void);
static void diag10_cancel_int(void);

static void diag11_test_smp(void);
static void diag12_test_exin(void);

static void isr_proc(int log_slot, void* arg);

static int    enter_log_slot(void);
static struct termio init_termio(void);
static void   term_termio(struct termio tty);


//--- ؿơ֥ ------------------------------------------

static void (*pdiag_func[])(void) =
{
	diag01_create,
	diag02_close,
	diag03_set_ad_md,
	diag04_start_ad,
	diag05_stop_ad,
	diag06_read_ad,
	diag07_get_ad_sts,
	diag08_get_exin,
	diag09_setup_int,
	diag10_cancel_int,
	diag11_test_smp,
	diag12_test_exin
};


int main(int argc, char *argv[])
{
	int cmd, i;

	printf("aPCI-8815 diagnostic utility.\n");

	do
	{
		printf("\n");
		printf("aPCI-8815 Test main menu\n");
		printf("------------------------\n");
		printf(" Function test          \n");	// ǽƥ
		printf("------------------------\n");
		printf(" 1. apci8815_create(int log_slot)                           \n");
		printf(" 2. apci8815_close(int log_slot)                            \n");
		printf(" 3. apci8815_set_ad_md(int log_slot, papci8815_admd pmd)    \n");
		printf(" 4. apci8815_start_ad(int log_slot)                         \n");
		printf(" 5. apci8815_stop_ad(int log_slot)                          \n");
		printf(" 6. apci8815_read_ad(int log_slot, unsigned short* buf, size_t count) \n");
		printf(" 7. apci8815_get_ad_sts(int log_slot, unsigned long* psts)  \n");
		printf(" 8. apci8815_get_exin(int log_slot, unsigned short* pexin)  \n");
		printf(" 9. apci8815_setup_int(int log_slot, void(*pcallback)(int, void*), void* arg)\n");
		printf("10. apci8815_cancel_int(int log_slot)                       \n");
		printf("------------------------\n");
		printf(" Application test       \n");	// ѥƥ
		printf("------------------------\n");
		printf("11. A/D Sampling                                           \n");
		printf("12. Outside input                                          \n");
		printf("------------------------\n");
		printf("99. Exit (All devices are closed)\n");
		printf("Enter option: ");

		cmd = 0;
		fgets(line, sizeof(line), stdin);
		sscanf(line, "%d", &cmd);

		if (cmd > 0 && cmd <= sizeof(pdiag_func) / sizeof(void*))
			pdiag_func[cmd - 1]();

		if(cmd != 99)
		{
			printf("--- Please enter key push! ---");
			fgets(line, sizeof(line), stdin);
		}
	}
	while (cmd != 99);

	// ƤΥǥХ򥯥
	for (i = 0; i < APCI8815_MAX_DEVICES; i++)
		apci8815_close(i);

	return 0;
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥץ							    */
/*--------------------------------------------------------------------------*/
static void diag01_create(void)
{
	int log_slot;

	printf("--- apci8815_create(int log_slot)\n");

	// Get LogSlot
	printf("Enter logslot(0-15 or -1): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	// Call Control Function
	log_slot = apci8815_create(log_slot);

	// result
	if (log_slot < 0)
		printf("Rslt: %d\n", log_slot);
	else
		printf("log_slot: %d\n", log_slot);
}

/*--------------------------------------------------------------------------*/
/*  ǥХΥ							    */
/*--------------------------------------------------------------------------*/
static void diag02_close(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8815_close(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8815_close(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  A/D ץ󥰾						    */
/*--------------------------------------------------------------------------*/
static void diag03_set_ad_md(void)
{
	int log_slot;
	apci8815_admd md;
	int ent, rslt;

	printf("--- apci8815_set_ad_md(int log_slot, papci8815_admd pmd)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Samplig mode
	printf("Enter sampling mode(1-4): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.md = (unsigned short)ent;

	// Get First channel
	printf("Enter first channel(0-15): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.ch_first = (char)ent;

	// Get Last channel
	printf("Enter last channel(0-15): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.ch_last = (char)ent;

	// Get Sampling cycle
	printf("Enter sampling cycle(12(45)-64000000): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.cycle = ent;

	// Get Buffer size
	printf("Enter buffer size(1-): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.buf_size = ent;

	// Call Control Function
	rslt = apci8815_set_ad_md(log_slot, &md);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  A/D ץ󥰳						    */
/*--------------------------------------------------------------------------*/
static void diag04_start_ad(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8815_start_ad(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8815_start_ad(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  A/D ץ						    */
/*--------------------------------------------------------------------------*/
static void diag05_stop_ad(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8815_stop_ad(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8815_stop_ad(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  A/D ץ󥰥ǡ					    */
/*--------------------------------------------------------------------------*/
static void diag06_read_ad(void)
{
	int log_slot, i, j;
	unsigned int count;
	ssize_t rslt_count;
	unsigned short* pbuf;

	printf("--- apci8815_read_ad(int log_slot, unsigned short* buf, size_t count)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Buffer size
	printf("Enter buffer count: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &count);

	pbuf = malloc(count * sizeof(unsigned short));
	if (pbuf == NULL)
	{
		printf("Memory allocation error!\n");
		return;
	}

	// Call Control Function
	rslt_count = apci8815_read_ad(log_slot, pbuf, count);

	if (rslt_count < 0)
	{
		printf("Rslt: %d\n", rslt_count);
		free(pbuf);
		return;
	}

	printf("Read count: %d\n", rslt_count);
	
	for (i = 0; i < rslt_count; i += 16)
	{
		for (j = i; j < i + 16 && j < rslt_count; j++)
			printf("%04X ", *(pbuf + j));

		printf("\n");
	}

	free(pbuf);
}

/*--------------------------------------------------------------------------*/
/*  A/D ơ						    */
/*--------------------------------------------------------------------------*/
static void diag07_get_ad_sts(void)
{
	int log_slot;
	unsigned long sts;
	int rslt;

	printf("--- apci8815_get_ad_sts(int log_slot, unsigned long* psts)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8815_get_ad_sts(log_slot, &sts);

	printf("Rslt: %d\n", rslt);
	printf("sts(hex): %x\n", (unsigned int)sts);
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡ					    */
/*--------------------------------------------------------------------------*/
static void diag08_get_exin(void)
{
	int log_slot;
	unsigned short exin;
	int rslt;

	printf("--- apci8815_get_exin(int log_slot, unsigned short* pexin)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Call Control Function
	rslt = apci8815_get_exin(log_slot, &exin);

	printf("Rslt: %d\n", rslt);
	printf("exin(hex): %x\n", exin);
}

/*--------------------------------------------------------------------------*/
/*  ߥХå롼					    */
/*--------------------------------------------------------------------------*/
static void diag09_setup_int(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8815_setup_int(int log_slot, void(*pcallback)(int, void*), void* arg)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	printf("\"pcallback\" and \"arg\" has been set up inside the program.\n\n");

	// Call Control Function
	rslt = apci8815_setup_int(log_slot, isr_proc, NULL);

	printf("Rslt: %d\n", rslt);
}


/*--------------------------------------------------------------------------*/
/*  ߥХå롼μä				    */
/*--------------------------------------------------------------------------*/
static void diag10_cancel_int(void)
{
	int log_slot;
	int rslt;

	printf("--- apci8815_cancel_int(int log_slot)\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	rslt = apci8815_cancel_int(log_slot);

	printf("Rslt: %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  A/D ץ󥰥ƥ						    */
/*--------------------------------------------------------------------------*/
static void diag11_test_smp(void)
{
	int log_slot, i, j, ent;
	apci8815_admd md;
	int rslt;
	char   c, n;
	struct termio tty_save;
	unsigned long sts;
	ssize_t rslt_count;
	unsigned short* pbuf;

	printf("--- The continuous test of A/D Sampling\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	// Get Samplig mode
	printf("Enter sampling mode(1-4): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.md = (unsigned short)ent;

	// Get First channel
	printf("Enter first channel(0-15): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.ch_first = (char)ent;

	// Get Last channel
	printf("Enter last channel(0-15): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.ch_last = (char)ent;

	// Get Sampling cycle
	printf("Enter sampling cycle(12(45)-64000000): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.cycle = ent;

	// Get Buffer size
	printf("Enter buffer size(1-): ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &ent);
	md.buf_size = ent;

	// Call Control Function
	rslt = apci8815_set_ad_md(log_slot, &md);
	if (rslt < 0)
	{
		printf("Rslt: %d\n", rslt);
		return;
	}

	pbuf = malloc(md.buf_size * sizeof(unsigned short));
	if (pbuf == NULL)
	{
		printf("Memory allocation error!\n");
		return;
	}

	sts = 0;

	// Call Control Function
	rslt = apci8815_setup_int(log_slot, isr_proc, &sts);
	if (rslt < 0)
	{
		printf("Rslt: %d\n", rslt);
		free(pbuf);
		return;
	}

	printf("\nA completion key is \"ESC\".\n\n");

	tty_save = init_termio();

	rslt = apci8815_start_ad(log_slot);
	if (rslt < 0)
	{
		term_termio(tty_save);
		printf("Rslt: %d\n", rslt);
		free(pbuf);
		return;
	}

	for (;;)
	{
		c = '\0';		// ʤ硢ƤѤǤޤ
		n = read(0, &c, 1);
		if (n == -1)
		{
			perror("read");
			break;
		}
		else if (c == 0x1B)	// ESC key
			break;

		if (sts == 0)
			continue;

		// Call Control Function
		rslt_count = apci8815_read_ad(log_slot, pbuf, md.buf_size);
		if (rslt_count < 0)
		{
			printf("Rslt: %d", rslt_count);
			printf("\r");
		}
		else
		{
			for (i = 0; i < rslt_count; i += 16)
			{
				for (j = i; j < i + 16 && j < rslt_count; j++)
					printf("%04X ", *(pbuf + j));

				printf("\n");
			}
		}

		sts = 0;

		if (md.md == APCI8815_AD_MD_01 || md.md == APCI8815_AD_MD_02)
			break;

		fflush(stdout);
	}

	// ץ
	rslt = apci8815_stop_ad(log_slot);
	if (rslt < 0)
		printf("Rslt: %d\n", rslt);

	// ᤹
	term_termio(tty_save);

	// Call Control Function
	rslt = apci8815_cancel_int(log_slot);
	if (rslt < 0)
		printf("Rslt: %d\n", rslt);

	free(pbuf);

	printf("\n");
}

/*--------------------------------------------------------------------------*/
/*  ϥݡȤǡϢ³					    */
/*--------------------------------------------------------------------------*/
static void diag12_test_exin(void)
{
	int log_slot;
	int rslt;
	char   c, n;
	struct termio tty_save;
	unsigned short exin;

	printf("--- The continuous test of Outside-input\n");

	// Get LogSlot
	log_slot = enter_log_slot();

	printf("\nA completion key is \"ESC\".\n\n");

	printf(" EXIN1 EXIN0 EXTRG\n");
	printf("-------------------\n");

	tty_save = init_termio();

	for (;;)
	{
		c = '\0';		// ʤ硢ƤѤǤޤ
		n = read(0, &c, 1);
		if (n == -1)
		{
			perror("read");
			break;
		}
		else if (c == 0x1B)	// ESC key
			break;

		rslt = apci8815_get_exin(log_slot, &exin);
		if (rslt < 0)
		{
			printf(" Rslt: %d ", rslt);
			printf("\r");
			fflush(stdout);
			continue;
		}

		// ExIn1 ξ
		printf("   %d  ", exin & APCI8815_STR_EXIN1 ? 0 : 1);

		// ExIn0 ξ
		printf("   %d  ", exin & APCI8815_STR_EXIN0 ? 0 : 1);

		// ExTrg ξ
		printf("   %d  ", exin & APCI8815_STR_EXTRG ? 0 : 1);

		printf("\r");
		fflush(stdout);
	}

	// ᤹
	term_termio(tty_save);

	printf("\n");
}

/*--------------------------------------------------------------------------*/
/*  ߥХå롼					    */
/*--------------------------------------------------------------------------*/
static void isr_proc(int log_slot, void* arg)
{
	int rslt;
	unsigned long* psts;

	if (arg == NULL)
		return;

	psts = (unsigned long*)arg;

	// Call Control Function
	rslt = apci8815_get_ad_sts(log_slot, psts);
	if (rslt < 0)
		printf("Rslt(sts): %d\n", rslt);
}

/*--------------------------------------------------------------------------*/
/*  åȤ							    */
/*--------------------------------------------------------------------------*/
static int enter_log_slot(void)
{
	int log_slot;

	printf("Enter log_slot: ");
	fgets(line, sizeof(line), stdin);
	sscanf(line, "%d", &log_slot);

	return log_slot;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७						    */
/*--------------------------------------------------------------------------*/
static struct termio init_termio(void)
{
	struct termio tty, tty_save;

	// ߤ¸
	ioctl(0, TCGETA, &tty);

	tty_save = tty;

	// ü
	tty.c_lflag &= ~(ECHO | ICANON);
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 2;
	ioctl(0, TCSETAF, &tty);

	return tty_save;
}

/*--------------------------------------------------------------------------*/
/*  ꥢ륿७ϲ						    */
/*--------------------------------------------------------------------------*/
static void term_termio(struct termio tty)
{
	// ᤹
	ioctl (0, TCSETAF, &tty);
}
